@@ -6,12 +6,12 @@ import unittest
 from distutils.version import LooseVersion
 
 import udiskstestcase
-from udiskstestcase import unstable_test
 
 
+# TODO: pull actual configured paths from the build
 MODPROBECONF = '/usr/lib/modprobe.d/zram.conf'
 MODLOADCONF = '/usr/lib/modules-load.d/zram.conf'
-ZRAMCONFDIR = '/usr/local/lib/zram.conf.d'
+ZRAMCONFDIR = '/usr/lib/zram.conf.d'
 
 
 class UdisksZRAMTest(udiskstestcase.UdisksTestCase):
@@ -31,7 +31,7 @@ class UdisksZRAMTest(udiskstestcase.UdisksTestCase):
         if os.path.exists(MODLOADCONF):
             cls.conf[MODLOADCONF] = cls.read_file(MODLOADCONF)
 
-        # /usr/local/lib/zram.conf.d
+        # /usr/lib/zram.conf.d
         if os.path.exists(ZRAMCONFDIR):
             for fname in os.listdir(ZRAMCONFDIR):
                 cls.conf[ZRAMCONFDIR][fname] = cls.read_file(ZRAMCONFDIR + '/' + fname)
@@ -46,21 +46,21 @@ class UdisksZRAMTest(udiskstestcase.UdisksTestCase):
         if cls.conf[MODLOADCONF]:
             cls.write_file(MODLOADCONF, cls.conf[MODLOADCONF])
 
-        # /usr/local/lib/zram.conf.d
+        # /usr/lib/zram.conf.d
         for fname in cls.conf[ZRAMCONFDIR].keys():
             cls.write_file(ZRAMCONFDIR + '/' + fname, cls.conf[ZRAMCONFDIR][fname])
 
     @classmethod
     def setUpClass(cls):
         udiskstestcase.UdisksTestCase.setUpClass()
-        if cls.distro[1:] == ("fedora", "27"):
-            raise unittest.SkipTest('zram module (un)loading causes kernel panics on rawhide')
-        if not cls.check_module_loaded('ZRAM'):
+        if not cls.check_module_loaded('zram'):
             udiskstestcase.UdisksTestCase.tearDownClass()
             raise unittest.SkipTest('Udisks module for zram tests not loaded, skipping.')
 
         cls._save_conf_files()
 
+        cls.run_command('systemctl preset udisks2-zram-setup@.service')
+
     @classmethod
     def tearDownClass(cls):
         udiskstestcase.UdisksTestCase.tearDownClass()
@@ -83,7 +83,14 @@ class UdisksZRAMTest(udiskstestcase.UdisksTestCase):
     def _swapoff(self, swap):
         self.run_command('swapoff %s' % swap)
 
-    @unstable_test
+    def test__manager_interface(self):
+        '''Test for module D-Bus Manager interface presence'''
+
+        manager = self.get_object('/Manager')
+        intro_data = manager.Introspect(self.no_options, dbus_interface='org.freedesktop.DBus.Introspectable')
+        self.assertIn('interface name="%s.Manager.ZRAM"' % self.iface_prefix, intro_data)
+
+    @udiskstestcase.tag_test(udiskstestcase.TestTags.UNSTABLE)
     def test_create_destroy(self):
         manager = self.get_object('/Manager')
         zrams = manager.CreateDevices([10 * 1024**2, 10 * 1024**2], [1, 2], self.no_options,
@@ -158,7 +165,7 @@ class UdisksZRAMTest(udiskstestcase.UdisksTestCase):
         dbus_orig = self.get_property(zram_obj, '.Block.ZRAM', 'OrigDataSize')
         dbus_orig.assertEqual(sys_orig)
 
-    @unstable_test
+    @udiskstestcase.tag_test(udiskstestcase.TestTags.UNSTABLE)
     def test_activate_deactivate(self):
         manager = self.get_object('/Manager')
         zrams = manager.CreateDevices([10 * 1024**2], [1], self.no_options,
