@@ -21,6 +21,7 @@
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
+#include <uuid.h>
 #include <blockdev/blockdev.h>
 
 #include "udiskslogging.h"
@@ -35,14 +36,18 @@
 #include "udisksthreadedjob.h"
 #include "udiskssimplejob.h"
 #include "udisksstate.h"
-#include "udisksfstabmonitor.h"
-#include "udisksfstabentry.h"
 #include "udiskscrypttabmonitor.h"
 #include "udiskscrypttabentry.h"
 #include "udiskslinuxblockobject.h"
 #include "udiskslinuxdevice.h"
 #include "udisksmodulemanager.h"
+#include "udisksmodule.h"
 #include "udisksconfigmanager.h"
+#include "udiskslinuxmountoptions.h"
+
+#ifdef HAVE_LIBMOUNT_UTAB
+#include "udisksutabmonitor.h"
+#endif
 
 /**
  * SECTION:udisksdaemon
@@ -75,10 +80,12 @@ struct _UDisksDaemon
 
   UDisksState *state;
 
-  UDisksFstabMonitor *fstab_monitor;
-
   UDisksCrypttabMonitor *crypttab_monitor;
 
+#ifdef HAVE_LIBMOUNT_UTAB
+  UDisksUtabMonitor *utab_monitor;
+#endif
+
   UDisksModuleManager *module_manager;
 
   UDisksConfigManager *config_manager;
@@ -86,6 +93,8 @@ struct _UDisksDaemon
   gboolean disable_modules;
   gboolean force_load_modules;
   gboolean uninstalled;
+  gboolean enable_tcrypt;
+  gchar *uuid;
 };
 
 struct _UDisksDaemonClass
@@ -99,13 +108,14 @@ enum
   PROP_CONNECTION,
   PROP_OBJECT_MANAGER,
   PROP_MOUNT_MONITOR,
-  PROP_FSTAB_MONITOR,
   PROP_CRYPTTAB_MONITOR,
   PROP_MODULE_MANAGER,
   PROP_CONFIG_MANAGER,
   PROP_DISABLE_MODULES,
   PROP_FORCE_LOAD_MODULES,
   PROP_UNINSTALLED,
+  PROP_ENABLE_TCRYPT,
+  PROP_UUID,
 };
 
 G_DEFINE_TYPE (UDisksDaemon, udisks_daemon, G_TYPE_OBJECT);
@@ -116,21 +126,24 @@ udisks_daemon_finalize (GObject *object)
   UDisksDaemon *daemon = UDISKS_DAEMON (object);
 
   udisks_state_stop_cleanup (daemon->state);
-  g_object_unref (daemon->state);
+
+  /* Modules use the monitors and try to reference them when cleaning up */
+  udisks_module_manager_unload_modules (daemon->module_manager);
 
   g_clear_object (&daemon->authority);
   g_object_unref (daemon->object_manager);
   g_object_unref (daemon->linux_provider);
   g_object_unref (daemon->connection);
-
-  /* Modules use the monitors and try to reference them when cleaning up */
-  udisks_module_manager_unload_modules (daemon->module_manager);
   g_object_unref (daemon->mount_monitor);
-  g_object_unref (daemon->fstab_monitor);
   g_object_unref (daemon->crypttab_monitor);
-
+#ifdef HAVE_LIBMOUNT_UTAB
+  g_object_unref (daemon->utab_monitor);
+#endif
   g_clear_object (&daemon->module_manager);
 
+  g_object_unref (daemon->state);
+  g_free (daemon->uuid);
+
   g_clear_object (&daemon->config_manager);
 
   if (G_OBJECT_CLASS (udisks_daemon_parent_class)->finalize != NULL)
@@ -159,10 +172,6 @@ udisks_daemon_get_property (GObject    *object,
       g_value_set_object (value, udisks_daemon_get_mount_monitor (daemon));
       break;
 
-    case PROP_FSTAB_MONITOR:
-      g_value_set_object (value, udisks_daemon_get_fstab_monitor (daemon));
-      break;
-
     case PROP_CRYPTTAB_MONITOR:
       g_value_set_object (value, udisks_daemon_get_crypttab_monitor (daemon));
       break;
@@ -187,6 +196,14 @@ udisks_daemon_get_property (GObject    *object,
       g_value_set_boolean (value, udisks_daemon_get_uninstalled (daemon));
       break;
 
+    case PROP_ENABLE_TCRYPT:
+      g_value_set_boolean (value, udisks_daemon_get_enable_tcrypt (daemon));
+      break;
+
+    case PROP_UUID:
+      g_value_set_string (value, udisks_daemon_get_uuid (daemon));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -200,6 +217,8 @@ udisks_daemon_set_property (GObject      *object,
                             GParamSpec   *pspec)
 {
   UDisksDaemon *daemon = UDISKS_DAEMON (object);
+  const gchar *s;
+  uuid_t uu;
 
   switch (prop_id)
     {
@@ -220,6 +239,21 @@ udisks_daemon_set_property (GObject      *object,
       daemon->uninstalled = g_value_get_boolean (value);
       break;
 
+    case PROP_ENABLE_TCRYPT:
+      daemon->enable_tcrypt = g_value_get_boolean (value);
+      break;
+
+    case PROP_UUID:
+      s = g_value_get_string (value);
+      if (s != NULL && uuid_parse (s, uu) == 0)
+        {
+          g_free (daemon->uuid);
+          daemon->uuid = g_strdup (s);
+        }
+      else
+        g_warning ("Invalid UUID string '%s'", s);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -240,12 +274,58 @@ mount_monitor_on_mount_removed (UDisksMountMonitor *monitor,
   udisks_state_check (daemon->state);
 }
 
+static gboolean
+load_modules_in_idle_cb (gpointer user_data)
+{
+  UDisksDaemon *daemon = UDISKS_DAEMON (user_data);
+
+  /* clear the state file, loading all modules anyway */
+  udisks_state_clear_modules (daemon->state);
+
+  udisks_module_manager_load_modules (daemon->module_manager);
+
+  return G_SOURCE_REMOVE;
+}
+
+static gboolean
+check_modules_state_in_idle_cb (gpointer user_data)
+{
+  UDisksDaemon *daemon = UDISKS_DAEMON (user_data);
+  gchar **modules;
+  gchar **l;
+  GError *error = NULL;
+
+  modules = udisks_state_get_modules (daemon->state);
+
+  /* first clear the state file */
+  udisks_state_clear_modules (daemon->state);
+
+  if (modules)
+    {
+      if (*modules)
+        g_warning ("Unclean shutdown detected, reloading modules from previous session.");
+
+      for (l = modules; l && *l; l++)
+        if (! udisks_module_manager_load_single_module (daemon->module_manager, *l, &error))
+          {
+            g_warning ("Error re-initializing module %s: %s", *l, error->message);
+            g_clear_error (&error);
+          }
+
+      g_strfreev (modules);
+    }
+
+  return G_SOURCE_REMOVE;
+}
+
 static void
 udisks_daemon_constructed (GObject *object)
 {
   UDisksDaemon *daemon = UDISKS_DAEMON (object);
   GError *error;
   gboolean ret = FALSE;
+  gchar uuid_buf[UUID_STR_LEN] = {0};
+  uuid_t uuid;
 
   /* NULL means no specific so_name (implementation) */
   BDPluginSpec part_plugin = {BD_PLUGIN_PART, NULL};
@@ -260,16 +340,45 @@ udisks_daemon_constructed (GObject *object)
      plugins are loaded themselves. */
   BDPluginSpec *plugins[] = {&part_plugin, &swap_plugin, &loop_plugin, &mdraid_plugin,
                              &fs_plugin, &crypto_plugin, NULL};
+  BDPluginSpec **plugin_p = NULL;
   error = NULL;
 
-  ret = bd_ensure_init (plugins, NULL, &error);
+  /* Skip runtime dependency checks when initializing libblockdev. Plugin
+     shouldn't fail to load just because some if its dependencies is missing.
+   */
+  ret = bd_switch_init_checks (FALSE, &error);
   if (!ret)
     {
       udisks_error ("Error initializing libblockdev library: %s (%s, %d)",
                     error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
     }
 
+  ret = bd_try_init (plugins, NULL, NULL, &error);
+  if (!ret)
+    {
+      if (error)
+        {
+          udisks_error ("Error initializing libblockdev library: %s (%s, %d)",
+                        error->message, g_quark_to_string (error->domain), error->code);
+          g_clear_error (&error);
+        }
+    else
+      {
+        /* a missing plugin is okay, calling functions from it will fail, but
+           until that happens, life will just be great */
+        for (plugin_p=plugins; *plugin_p; plugin_p++)
+          if (!bd_is_plugin_available ((*plugin_p)->name))
+            udisks_warning ("Failed to load the '%s' libblockdev plugin",
+                            bd_get_plugin_name ((*plugin_p)->name));
+      }
+    }
+
+  /* Generate global UUID */
+  uuid_generate (uuid);
+  uuid_unparse (uuid, &uuid_buf[0]);
+  daemon->uuid = g_strdup (uuid_buf);
+
   daemon->authority = polkit_authority_get_sync (NULL, &error);
   if (daemon->authority == NULL)
     {
@@ -317,21 +426,34 @@ udisks_daemon_constructed (GObject *object)
                     G_CALLBACK (mount_monitor_on_mount_removed),
                     daemon);
 
-  daemon->fstab_monitor = udisks_fstab_monitor_new ();
   daemon->crypttab_monitor = udisks_crypttab_monitor_new ();
+#ifdef HAVE_LIBMOUNT_UTAB
+  daemon->utab_monitor = udisks_utab_monitor_new ();
+#endif
 
   /* now add providers */
   daemon->linux_provider = udisks_linux_provider_new (daemon);
-
-  if (daemon->force_load_modules
-      || (udisks_config_manager_get_load_preference (daemon->config_manager)
-          == UDISKS_MODULE_LOAD_ONSTARTUP))
-    {
-      udisks_module_manager_load_modules (daemon->module_manager);
-    }
-
   udisks_provider_start (UDISKS_PROVIDER (daemon->linux_provider));
 
+  /* fill in default mount options */
+  g_object_set_data_full (object,
+                          "mount-options",
+                          udisks_linux_mount_options_get_builtin (),
+                          (GDestroyNotify) g_hash_table_destroy);
+
+  /* Load modules if requested but only once providers have started and
+   * have connected on the UDisksModuleManager::modules-activated signal.
+   */
+  if (daemon->force_load_modules ||
+      udisks_config_manager_get_load_preference (daemon->config_manager) == UDISKS_MODULE_LOAD_ONSTARTUP)
+    {
+      /* load all modules */
+      g_idle_add (load_modules_in_idle_cb, daemon);
+    } else {
+      /* crash recovery - load active modules from previous session */
+      g_idle_add (check_modules_state_in_idle_cb, daemon);
+    }
+
   /* Export the ObjectManager */
   g_dbus_object_manager_server_set_connection (daemon->object_manager, daemon->connection);
 
@@ -443,6 +565,35 @@ udisks_daemon_class_init (UDisksDaemonClass *klass)
                                                          G_PARAM_READABLE |
                                                          G_PARAM_WRITABLE |
                                                          G_PARAM_CONSTRUCT_ONLY));
+
+  /**
+   * UDisksDaemon:enable-tcrypt:
+   *
+   * Whether devices should be tested for being TCRYPT encrypted.
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_ENABLE_TCRYPT,
+                                   g_param_spec_boolean ("enable-tcrypt",
+                                                         "Enable TCRYPT",
+                                                         "Whether devices should be tested for being TCRYPT encrypted",
+                                                         FALSE,
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_WRITABLE |
+                                                         G_PARAM_CONSTRUCT_ONLY));
+
+  /**
+   * UDisksDaemon:uuid:
+   *
+   * The UUID specific for this daemon instance.
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_UUID,
+                                   g_param_spec_string ("uuid",
+                                                        "Daemon UUID",
+                                                        "The UUID specific for this daemon instance",
+                                                         NULL,
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_WRITABLE));
 }
 
 /**
@@ -451,6 +602,7 @@ udisks_daemon_class_init (UDisksDaemonClass *klass)
  * @disable_modules: Indicates whether modules should never be activated.
  * @force_load_modules: Activate modules on startup (for debugging purposes).
  * @uninstalled: Loads modules from the build directory (for debugging purposes).
+ * @enable_tcrypt: Checks whether devices could be TCRYPT encrypted.
  *
  * Create a new daemon object for exporting objects on @connection.
  *
@@ -460,7 +612,8 @@ UDisksDaemon *
 udisks_daemon_new (GDBusConnection *connection,
                    gboolean         disable_modules,
                    gboolean         force_load_modules,
-                   gboolean         uninstalled)
+                   gboolean         uninstalled,
+                   gboolean         enable_tcrypt)
 {
   g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), NULL);
   return UDISKS_DAEMON (g_object_new (UDISKS_TYPE_DAEMON,
@@ -468,6 +621,7 @@ udisks_daemon_new (GDBusConnection *connection,
                                       "disable-modules", disable_modules,
                                       "force-load-modules", force_load_modules,
                                       "uninstalled", uninstalled,
+                                      "enable-tcrypt", enable_tcrypt,
                                       NULL));
 }
 
@@ -516,21 +670,6 @@ udisks_daemon_get_mount_monitor (UDisksDaemon *daemon)
   return daemon->mount_monitor;
 }
 
-/**
- * udisks_daemon_get_fstab_monitor:
- * @daemon: A #UDisksDaemon
- *
- * Gets the fstab monitor used by @daemon.
- *
- * Returns: A #UDisksFstabMonitor. Do not free, the object is owned by @daemon.
- */
-UDisksFstabMonitor *
-udisks_daemon_get_fstab_monitor (UDisksDaemon *daemon)
-{
-  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
-  return daemon->fstab_monitor;
-}
-
 /**
  * udisks_daemon_get_crypttab_monitor:
  * @daemon: A #UDisksDaemon
@@ -546,6 +685,23 @@ udisks_daemon_get_crypttab_monitor (UDisksDaemon *daemon)
   return daemon->crypttab_monitor;
 }
 
+#ifdef HAVE_LIBMOUNT_UTAB
+/**
+ * udisks_daemon_get_utab_monitor:
+ * @daemon: A #UDisksDaemon
+ *
+ * Gets the utab monitor used by @daemon.
+ *
+ * Returns: A #UDisksUtabMonitor. Do not free, the object is owned by @daemon.
+ */
+UDisksUtabMonitor *
+udisks_daemon_get_utab_monitor (UDisksDaemon *daemon)
+{
+  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
+  return daemon->utab_monitor;
+}
+#endif
+
 /**
  * udisks_daemon_get_linux_provider:
  * @daemon: A #UDisksDaemon.
@@ -595,13 +751,28 @@ udisks_daemon_get_state (UDisksDaemon *daemon)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+typedef struct
+{
+  UDisksDaemon *daemon;
+  UDisksInhibitCookie *inhibit_cookie;
+} JobData;
+
+static void
+free_job_data (JobData *job_data)
+{
+  if (job_data->daemon != NULL)
+    g_object_unref (job_data->daemon);
+  g_free (job_data);
+}
+
 static void
 on_job_completed (UDisksJob    *job,
                   gboolean      success,
                   const gchar  *message,
                   gpointer      user_data)
 {
-  UDisksDaemon *daemon = UDISKS_DAEMON (user_data);
+  JobData *job_data = (JobData *) user_data;
+  UDisksDaemon *daemon = UDISKS_DAEMON (job_data->daemon);
   UDisksObjectSkeleton *object;
 
   object = UDISKS_OBJECT_SKELETON (g_dbus_interface_get_object (G_DBUS_INTERFACE (job)));
@@ -615,11 +786,13 @@ on_job_completed (UDisksJob    *job,
   /* free the allocated job object */
   g_object_unref (job);
 
+  /* unregister inhibitor from systemd logind */
+  udisks_daemon_util_uninhibit_system_sync (job_data->inhibit_cookie);
   /* returns the reference we took when connecting to the
    * UDisksJob::completed signal in udisks_daemon_launch_{spawned,threaded}_job()
    * below
    */
-  g_object_unref (daemon);
+  free_job_data (job_data);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -637,6 +810,15 @@ common_job (UDisksDaemon    *daemon,
 {
   gchar *job_object_path;
   UDisksObjectSkeleton *job_object;
+  JobData *job_data;
+  gchar *operation_description;
+
+  job_data = g_new0 (JobData, 1);
+  job_data->daemon = g_object_ref (daemon);
+  /* register inhibitor to systemd logind while job is running */
+  operation_description = udisks_client_get_job_description_from_operation (job_operation);
+  job_data->inhibit_cookie = udisks_daemon_util_inhibit_system_sync (operation_description);
+  g_free (operation_description);
 
   if (object != NULL)
     udisks_base_job_add_object (UDISKS_BASE_JOB (job), object);
@@ -654,7 +836,7 @@ common_job (UDisksDaemon    *daemon,
   g_signal_connect_after (job,
                           "completed",
                           G_CALLBACK (on_job_completed),
-                          g_object_ref (daemon));
+                          job_data);
 
   return UDISKS_BASE_JOB (job);
 }
@@ -813,6 +995,7 @@ udisks_daemon_launch_spawned_job (UDisksDaemon    *daemon,
                                           command_line);
 
   udisks_string_wipe_and_free (input_string_as_gstring);
+  g_free (command_line);
   return job;
 }
 
@@ -975,6 +1158,7 @@ udisks_daemon_launch_spawned_job_sync (UDisksDaemon    *daemon,
                                           command_line);
 
   udisks_string_wipe_and_free (input_string_as_gstring);
+  g_free (command_line);
   return ret;
 }
 
@@ -1077,37 +1261,41 @@ udisks_daemon_launch_spawned_job_gstring_sync (UDisksDaemon    *daemon,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-typedef struct
-{
-  GMainContext *context;
-  GMainLoop    *loop;
-  gboolean      result;
-  GError       *error;
-} ThreadedJobSyncData;
-
-static gboolean
-threaded_job_sync_on_threaded_job_completed (UDisksThreadedJob *job,
-                                             gboolean           result,
-                                             GError            *error,
-                                             gpointer           user_data)
-{
-  ThreadedJobSyncData *data = user_data;
-  data->result = result;
-  if (error)
-    data->error = g_error_copy (error);
-  return FALSE; /* let other handlers run */
-}
+static __thread UDisksJob *thread_job = NULL;
 
 static void
-threaded_job_sync_on_completed (UDisksJob    *job,
-                                gboolean      success,
-                                const gchar  *message,
-                                gpointer      user_data)
+bd_thread_progress_callback (guint64            task_id,
+                             BDUtilsProgStatus  status,
+                             guint8             completion,
+                             gchar             *msg)
 {
-  ThreadedJobSyncData *data = user_data;
-  g_main_loop_quit (data->loop);
+  if (thread_job != NULL && msg == NULL)
+    {
+      if (!udisks_job_get_progress_valid (UDISKS_JOB (thread_job)))
+        {
+          udisks_job_set_progress_valid (UDISKS_JOB (thread_job), TRUE);
+        }
+
+      udisks_job_set_progress (UDISKS_JOB (thread_job), completion / 100.0);
+    }
 }
 
+void
+udisks_bd_thread_set_progress_for_job (UDisksJob *job)
+{
+  thread_job = job;
+  bd_utils_init_prog_reporting_thread (bd_thread_progress_callback, NULL);
+}
+
+void
+udisks_bd_thread_disable_progress (void)
+{
+  thread_job = NULL;
+  bd_utils_init_prog_reporting_thread (NULL, NULL);
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 /**
  * udisks_daemon_launch_threaded_job_sync:
  * @daemon: A #UDisksDaemon.
@@ -1137,17 +1325,11 @@ udisks_daemon_launch_threaded_job_sync (UDisksDaemon          *daemon,
                                         GError               **error)
 {
   UDisksBaseJob *job;
-  ThreadedJobSyncData data;
+  gboolean job_result;
 
   g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), FALSE);
   g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
 
-  data.context = g_main_context_new ();
-  g_main_context_push_thread_default (data.context);
-  data.loop = g_main_loop_new (data.context, FALSE);
-  data.error = NULL;
-  data.result = FALSE;
-
   job = udisks_daemon_launch_threaded_job (daemon,
                                            object,
                                            job_operation,
@@ -1156,27 +1338,14 @@ udisks_daemon_launch_threaded_job_sync (UDisksDaemon          *daemon,
                                            user_data,
                                            user_data_free_func,
                                            cancellable);
-  g_signal_connect (job,
-                    "threaded-job-completed",
-                    G_CALLBACK (threaded_job_sync_on_threaded_job_completed),
-                    &data);
-  g_signal_connect_after (job,
-                          "completed",
-                          G_CALLBACK (threaded_job_sync_on_completed),
-                          &data);
 
-  udisks_threaded_job_start (UDISKS_THREADED_JOB (job));
-  g_main_loop_run (data.loop);
-
-  g_main_loop_unref (data.loop);
-  g_main_context_pop_thread_default (data.context);
-  g_main_context_unref (data.context);
-
-  if (data.error)
-    g_propagate_error (error, data.error);
+  /* TODO: There might not be much difference between calling the job_func() right away
+   *       instead of having it enclosed in a GTask since we're blocking anyway.
+   *       Deeper investigation of differences required. */
+  job_result = udisks_threaded_job_run_sync (UDISKS_THREADED_JOB (job), error);
 
   /* note: the job object is freed in the ::completed handler */
-  return data.result;
+  return job_result;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1275,6 +1444,8 @@ static gpointer wait_for_objects (UDisksDaemon                *daemon,
         }
       else
         {
+          if (to_disappear)
+            g_object_unref (G_OBJECT (ret));
           goto again;
         }
     }
@@ -1378,7 +1549,9 @@ udisks_daemon_wait_for_objects_sync (UDisksDaemon                 *daemon,
  * %UDISKS_ERROR_TIMED_OUT).
  *
  * Note that @wait_func will be called from time to time - for example
- * if there is a device event.
+ * if there is a device event. For consistency @wait_func is supposed
+ * to return full reference to an existing object; udisks_daemon_wait_for_object_to_disappear_sync()
+ * will take care of dropping the reference after each iteration.
  *
  * Returns: (transfer full): Whether the object picked by @wait_func disappeared or not (@error is set).
  */
@@ -1390,13 +1563,19 @@ udisks_daemon_wait_for_object_to_disappear_sync (UDisksDaemon               *dae
                                                  guint                       timeout_seconds,
                                                  GError                      **error)
 {
-  return NULL == wait_for_objects (daemon,
-                                   (UDisksDaemonWaitFuncGeneric) wait_func,
-                                   user_data,
-                                   user_data_free_func,
-                                   timeout_seconds,
-                                   TRUE, /* to_disappear */
-                                   error);
+  UDisksObject *object;
+
+  object = (UDisksObject *) wait_for_objects (daemon,
+                                              (UDisksDaemonWaitFuncGeneric) wait_func,
+                                              user_data,
+                                              user_data_free_func,
+                                              timeout_seconds,
+                                              TRUE, /* to_disappear */
+                                              error);
+  if (object != NULL)
+    g_object_unref (object);
+
+  return NULL == object;
 }
 
 
@@ -1576,7 +1755,7 @@ udisks_daemon_get_module_manager (UDisksDaemon *daemon)
  *
  * Gets the config manager used by @daemon.
  *
- * Returns: A #ConfigModuleManager. Do not free, the object is owned by @daemon.
+ * Returns: A #UDisksConfigManager. Do not free, the object is owned by @daemon.
  */
 UDisksConfigManager *
 udisks_daemon_get_config_manager (UDisksDaemon *daemon)
@@ -1631,12 +1810,55 @@ udisks_daemon_get_uninstalled (UDisksDaemon *daemon)
   return daemon->uninstalled;
 }
 
+/**
+ * udisks_daemon_get_enable_tcrypt:
+ * @daemon: A #UDisksDaemon.
+ *
+ * Gets @daemon setting whether devices should be tested for being TCRYPT
+ * encrypted.
+ *
+ * Returns: %TRUE if "/etc/udisks2/tcrypt.conf" was an existing file during
+ * daemon startup.
+ */
+gboolean
+udisks_daemon_get_enable_tcrypt (UDisksDaemon *daemon)
+{
+  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), FALSE);
+  return daemon->enable_tcrypt;
+}
+
+/**
+ * udisks_daemon_get_uuid:
+ * @daemon: A #UDisksDaemon.
+ *
+ * Gets the UUID string specific to this @daemon instance.
+ *
+ * Returns: the UUID string. Do not free, the string is owned by @daemon.
+ */
+const gchar *
+udisks_daemon_get_uuid (UDisksDaemon *daemon)
+{
+  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
+  return daemon->uuid;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
+/**
+ * udisks_daemon_get_parent_for_tracking:
+ * @daemon: A #UDisksDaemon.
+ * @path: object path of a child to find parent of
+ * @uuid: a pointer to return parent UUID string
+ *
+ * Finds parent block device and returns its object path and UUID.
+ * If the return value is %NULL, the value of @uuid has not been changed.
+ *
+ * Returns: (transfer full): object path of the parent device. Free with g_free().
+ */
 gchar *
 udisks_daemon_get_parent_for_tracking (UDisksDaemon  *daemon,
                                        const gchar   *path,
-                                       gchar        **uuid_ret)
+                                       gchar        **uuid)
 {
   const gchar *parent_path = NULL;
   const gchar *parent_uuid = NULL;
@@ -1645,7 +1867,6 @@ udisks_daemon_get_parent_for_tracking (UDisksDaemon  *daemon,
   UDisksObject *crypto_object = NULL;
   UDisksObject *mdraid_object = NULL;
   UDisksObject *table_object = NULL;
-  GList *track_parent_funcs;
 
   UDisksBlock *block;
   UDisksBlock *crypto_block;
@@ -1653,6 +1874,10 @@ udisks_daemon_get_parent_for_tracking (UDisksDaemon  *daemon,
   UDisksPartition *partition;
   UDisksBlock *table_block;
 
+  GList *modules;
+  GList *l;
+  gchar *path_ret = NULL;
+
   object = udisks_daemon_find_object (daemon, path);
   if (object == NULL)
     goto out;
@@ -1713,21 +1938,24 @@ udisks_daemon_get_parent_for_tracking (UDisksDaemon  *daemon,
 
   if (parent_path)
     {
-      if (uuid_ret)
-        *uuid_ret = g_strdup (parent_uuid);
+      if (uuid)
+        *uuid = g_strdup (parent_uuid);
       return g_strdup (parent_path);
     }
 
-  track_parent_funcs = udisks_module_manager_get_track_parent_funcs (daemon->module_manager);
-  while (track_parent_funcs)
+  modules = udisks_module_manager_get_modules (daemon->module_manager);
+  for (l = modules; l != NULL; l = l->next)
     {
-      UDisksTrackParentFunc func = track_parent_funcs->data;
-      gchar *path_ret = func (daemon, path, uuid_ret);
-      if (path_ret)
-        return path_ret;
+      UDisksModule *module = l->data;
 
-      track_parent_funcs = track_parent_funcs->next;
+      path_ret = udisks_module_track_parent (module, path, uuid);
+      if (path_ret)
+        break;
     }
+  g_list_free_full (modules, g_object_unref);
+
+  if (path_ret)
+    return path_ret;
 
   return NULL;
 }
